# ruff: noqa: D402,PLR0913
# pyright: reportConstantRedefinition=none
"""The script commands module.

These commands are used in the following models:
SS3706A

THIS FILE IS AUTO-GENERATED, IT SHOULD NOT BE MANUALLY MODIFIED.

Please report an issue if one is found.

Attributes and Functions:

::

    - script.anonymous
    - script.delete()
    - script.load()
    - script.new()
    - script.newautorun()
    - script.restore()
    - script.run()
"""
from typing import Optional, TYPE_CHECKING

from .._helpers import BaseTSPCmd, NoDeviceProvidedError

if TYPE_CHECKING:
    from tm_devices.drivers.pi.tsp_device import TSPDevice


class Script(BaseTSPCmd):
    """The ``script`` command tree.

    Properties/methods:
        - ``.anonymous``: The ``script.anonymous`` attribute.
        - ``.delete()``: The ``script.delete()`` function.
        - ``.load()``: The ``script.load()`` function.
        - ``.new()``: The ``script.new()`` function.
        - ``.newautorun()``: The ``script.newautorun()`` function.
        - ``.restore()``: The ``script.restore()`` function.
        - ``.run()``: The ``script.run()`` function.
    """

    def __init__(self, device: Optional["TSPDevice"] = None, cmd_syntax: str = "script") -> None:
        super().__init__(device, cmd_syntax)

    @property
    def anonymous(self) -> str:
        """Access the ``script.anonymous`` attribute.

        **Description:**
            - This is a reference to the anonymous script.

        **Usage:**
            - Accessing this property will send the ``print(script.anonymous)`` query.

        **TSP Syntax:**

        ::

            - print(script.anonymous)

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            if self._device.command_syntax_enabled:  # type: ignore[union-attr]
                return self._cmd_syntax + ".anonymous"
            return self._device.query(  # type: ignore[union-attr]
                f"print({self._cmd_syntax}.anonymous)"
            )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to access the ``{self._cmd_syntax}.anonymous`` attribute."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    def delete(self, script_name: str) -> None:
        """Run the ``script.delete()`` function.

        **Description:**
            - This function deletes a script from the runtime memory and nonvolatile memory.

        **TSP Syntax:**

        ::

            - script.delete()

        Args:
            script_name: A string that represents the name of the script.

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            self._device.write(  # type: ignore[union-attr]
                f'{self._cmd_syntax}.delete("{script_name}")'
            )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to run the ``{self._cmd_syntax}.delete()`` function."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    def load(self, file: str) -> str:
        """Run the ``script.load()`` function.

        **Description:**
            - This function creates a script from a specified file.

        **TSP Syntax:**

        ::

            - script.load()

        Args:
            file: A string that contains the path and file name of the script file to load; if
                scriptVar is not defined, this name is used as the global variable name for this
                script.

        Returns:
            The result of the function call.

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            return self._device.query(  # type: ignore[union-attr]
                f'print({self._cmd_syntax}.load("{file}"))'
            )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to run the ``{self._cmd_syntax}.load()`` function."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    def new(self, code: str, name: Optional[str] = None) -> str:
        """Run the ``script.new()`` function.

        **Description:**
            - This function creates a script.

        **TSP Syntax:**

        ::

            - script.new()

        Args:
            code: A string containing the body of the script.
            name (optional): The name of the script.

        Returns:
            The result of the function call.

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            function_args = ", ".join(
                str(x)
                for x in (
                    f'"{code}"',
                    f'"{name}"' if name is not None else None,
                )
                if x is not None
            )
            return self._device.query(  # type: ignore[union-attr]
                f"print({self._cmd_syntax}.new({function_args}))"
            )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to run the ``{self._cmd_syntax}.new()`` function."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    def newautorun(self, code: str, name: Optional[str] = None) -> str:
        """Run the ``script.newautorun()`` function.

        **Description:**
            - This function creates a script and enables autorun.

        **TSP Syntax:**

        ::

            - script.newautorun()

        Args:
            code: A string that contains the body of the script.
            name (optional): The name of the script.

        Returns:
            The result of the function call.

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            function_args = ", ".join(
                str(x)
                for x in (
                    f'"{code}"',
                    f'"{name}"' if name is not None else None,
                )
                if x is not None
            )
            return self._device.query(  # type: ignore[union-attr]
                f"print({self._cmd_syntax}.newautorun({function_args}))"
            )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to run the ``{self._cmd_syntax}.newautorun()`` function."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    def restore(self, name: str) -> None:
        """Run the ``script.restore()`` function.

        **Description:**
            - This function restores a script that was removed from the runtime environment.

        **TSP Syntax:**

        ::

            - script.restore()

        Args:
            name: The name of the script to be restored.

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            self._device.write(f"{self._cmd_syntax}.restore({name})")  # type: ignore[union-attr]
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to run the ``{self._cmd_syntax}.restore()`` function."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    def run(self) -> None:
        """Run the ``script.run()`` function.

        **Description:**
            - This function runs the anonymous script.

        **TSP Syntax:**

        ::

            - script.run()

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            self._device.write(f"{self._cmd_syntax}.run()")  # type: ignore[union-attr]
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to run the ``{self._cmd_syntax}.run()`` function."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error



"""DPO70KDX device driver module."""
from tm_devices.commands import DPO70KDXMixin
from tm_devices.drivers.pi.scopes.tekscope_5k_7k_70k.dpo70k import DPO70K


class DPO70KDX(DPO70KDXMixin, DPO70K):
    """DPO70KDX device driver."""



"""Module containing constants and dataclasses for the tm_devices package."""
import re

from dataclasses import dataclass
from types import MappingProxyType
from typing import Final, List, Mapping, Optional, Protocol, runtime_checkable

import pyvisa as visa

from tm_devices.helpers.dataclass_mixins import (
    AsDictionaryMixin,
    AsDictionaryUseEnumNameUseCustEnumStrValueMixin,
)
from tm_devices.helpers.enums import ConnectionTypes, DeviceTypes, SupportedModels
from tm_devices.helpers.standalone_functions import validate_address


@runtime_checkable
@dataclass
class DataclassProtocol(Protocol):
    """A Protocol class to allow for type hinting things that accept generic dataclasses."""


@dataclass
class USBTMCConfiguration:
    """Dataclass for holding USBTMC configuration information.

    Each instance of the dataclass holds a Vendor ID and a Model ID to allow easier creation of
    USBTMC VISA resource expressions.
    """

    vendor_id: str
    """str: The hexadecimal Vendor ID used to create the USBTMC resource expression."""
    model_id: str
    """str: The hexadecimal Model ID used to create the USBTMC resource expression."""


class _ConfigEntryEnvStrMixin(AsDictionaryMixin):
    """Implements __str__ dunder method to format configuration information as an env variable.

    Handles nested classes through a prefix mapping stored in CONFIG_CLASS_STR_PREFIX_MAPPING.
    Changes to the mapping need to be reversible in the dm_config_parser which needs to bundle
    prefixed values under their own dictionary (to mimic the return type from the toml/yaml config
    files -> ``Dict[str, Union[str, int, Dict[str,...]]]``).

    So make sure to also modify DMConfigParser's _CONFIG_NESTED_DICT_MAPPING class property.
    """

    def __str__(self) -> str:
        """Return a single line of comma separated, key-value pairs for an environment variable."""
        ret_list: List[str] = []
        for key, val in self.to_dict(ignore_none=True).items():
            # must pull class type from actual value, not the to_dict()'s representation
            if (
                prefix := CONFIG_CLASS_STR_PREFIX_MAPPING.get(type(getattr(self, key)))
            ) is not None:
                if any(val.values()):
                    # still need to ignore the None values
                    str_pair = ",".join(
                        f"{prefix}{key2}={val2}" for key2, val2 in val.items() if val2 is not None
                    )
                else:
                    continue  # pragma: no cover
            else:
                str_pair = f"{key}={val}"
            ret_list.append(str_pair)
        return ",".join(ret_list)


@dataclass
class SerialConfig(AsDictionaryUseEnumNameUseCustEnumStrValueMixin, _ConfigEntryEnvStrMixin):
    """Serial configuration properties for connecting to a device over SERIAL (ASRL).

    Notes:
        - baud_rate: The baud rate controls the communication frequency.
            e.g. 9600
        - data_bits: The number of data bits in each character.
            One of [5, 6, 7, 8].
        - flow_control: Control for pausing/resuming data stream between slower devices.
            One of ``SerialConfig.FlowControl.[none|xon_xoff|dtr_dsr|rts_cts]``.
        - parity: Parity controls checksum bit (added to each data character) behavior.
            One of ``SerialConfig.Parity.[none|odd|even|mark|space]``.
        - stop_bits: Number of bits to use to indicate end of a character.
            One of ``SerialConfig.StopBits.[one|one_and_a_half|two]``.
        - end_input: Character(s) to indicate the end of a message transmission.
            One of ``SerialConfig.Termination.[termination_break|termination_char|last_bit|none]``.
    """

    FlowControl = visa.resources.serial.constants.ControlFlow
    """visa.resources.serial.constants.ControlFlow: Control flow for a serial device."""
    Parity = visa.resources.serial.constants.Parity
    """visa.resources.serial.constants.Parity: Parity type to use with every frame transmitted and received on a serial session.

    Used only for SERIAL (ASRL) resources.
    """  # noqa: E501
    StopBits = visa.resources.serial.constants.StopBits
    """visa.resources.serial.constants.StopBits: The number of stop bits that indicate the end of a frame on a serial resource.

    Used only for SERIAL (ASRL) resources.
    """  # noqa: E501
    Termination = visa.resources.serial.constants.SerialTermination
    """visa.resources.serial.constants.SerialTermination: The available methods for terminating a serial transfer."""  # noqa: E501
    baud_rate: Optional[int] = None
    """Optional[int]: The baud rate controls the communication frequency."""
    data_bits: Optional[int] = None
    """Optional[int]: The number of data bits in each character.

    One of ``[5, 6, 7, 8]``.
    """
    flow_control: Optional[FlowControl] = None
    """Optional[FlowControl]: Control for pausing/resuming data stream between slower devices.

    One of ``SerialConfig.FlowControl.[none|xon_xoff|dtr_dsr|rts_cts]``."""
    parity: Optional[Parity] = None
    """Optional[Parity]: Parity adds a checksum bit to each data character.
     Checksum bit enables the target device to determine whether the data was received correctly.

    One of ``SerialConfig.Parity.[none|odd|even|mark|space]``.
    """
    stop_bits: Optional[StopBits] = None
    """Optional[StopBits]: Number of bits to use to indicate end of a character.

    One of ``SerialConfig.StopBits.[one|one_and_a_half|two]``.
    """
    end_input: Optional[Termination] = None
    """Optional[Termination]: Character(s) to indicate the end of a message transmission.

    One of ``SerialConfig.Termination.[termination_break|termination_char|last_bit|none]``.
    """

    def __post_init__(self) -> None:
        """Convert from string to expected type and translate enum string names into the values."""
        if isinstance(self.baud_rate, str):
            self.baud_rate = int(self.baud_rate)
        if isinstance(self.data_bits, str):
            self.data_bits = int(self.data_bits)
        if isinstance(self.flow_control, str):
            self.flow_control = getattr(self.FlowControl, self.flow_control)
        if isinstance(self.parity, str):
            self.parity = getattr(self.Parity, self.parity)
        if isinstance(self.stop_bits, str):
            self.stop_bits = getattr(self.StopBits, self.stop_bits)
        if isinstance(self.end_input, str):
            self.end_input = getattr(self.Termination, self.end_input)


@dataclass(frozen=True)
class DeviceConfigEntry(AsDictionaryUseEnumNameUseCustEnumStrValueMixin, _ConfigEntryEnvStrMixin):
    """Dataclass for holding configuration information for a single device."""

    device_type: DeviceTypes
    """tm_devices.helpers.DeviceTypes: The specific device type defined in the config entry."""
    address: str
    """str: The address used to connect to the device.

    Notes:
        - TCPIP: IP address or the hostname.
        - SOCKET: IP address or the hostname (must define `lan_port` field).
        - REST_API: IP address or the hostname (must define `lan_port` field).
        - SERIAL/ASRL: serial COM port number.
        - USB: use expression format f"{model}-{serial_number}" (ex: "MSO24-ABC0123").
    """
    connection_type: ConnectionTypes = ConnectionTypes.TCPIP
    """tm_devices.helpers.ConnectionTypes: The specific type of connection defined in the config entry."""  # noqa: E501
    alias: Optional[str] = None
    """Optional[str]: An optional key/name used to retrieve this devices from the DeviceManager."""
    lan_port: Optional[int] = None
    """Optional[int]: The port number to connect on, used for SOCKET/REST_API connections."""
    serial_config: Optional[SerialConfig] = None
    """Optional[SerialConfig]: Serial configuration properties for connecting to a device over SERIAL (ASRL)."""  # noqa: E501
    device_driver: Optional[str] = None
    """Optional[str]: The name of the Python driver to use for the device (required for connection_type=REST_API, ignored otherwise)"""  # noqa: E501

    # pylint: disable=too-many-branches
    def __post_init__(self) -> None:  # noqa: PLR0912,C901
        """Validate data after creation.

        Raises:
            TypeError: Indicates a bad enum type.
            ValueError: Indicates a bad input value.
        """
        # manually handle type conversion from parsing from config file
        # (this is the correct way to edit the frozen data)
        if self.lan_port is not None and isinstance(self.lan_port, str):
            object.__setattr__(self, "lan_port", int(self.lan_port))
        if not isinstance(self.address, str):  # pyright: ignore
            object.__setattr__(self, "address", str(self.address))

        # Validate the connection and interface types
        try:
            # device and connections are inside this try clause to raise a TypeError on bad values.
            # Convert them from strings (parsed from config/env) to appropriate enum value.
            if not isinstance(self.device_type, DeviceTypes):  # pyright: ignore
                object.__setattr__(self, "device_type", DeviceTypes(self.device_type))
            if not isinstance(self.connection_type, ConnectionTypes):  # pyright: ignore
                object.__setattr__(self, "connection_type", ConnectionTypes(self.connection_type))

            # While a SerialConfig is not frozen, if not created here then it cannot be added later.
            # A serial_config must be created if the connection_type is SERIAL (ASRL).
            if self.connection_type == ConnectionTypes.SERIAL and self.serial_config is None:
                object.__setattr__(self, "serial_config", SerialConfig())

            if self.device_type not in VALID_DEVICE_CONNECTION_TYPES:
                msg = (
                    f"{self.device_type.name} is not a valid device type. Valid device types: "
                    f"{VALID_DEVICE_CONNECTION_TYPES}"
                )
                raise TypeError(msg)
            if self.connection_type not in VALID_DEVICE_CONNECTION_TYPES[self.device_type]:
                msg = (
                    f"{self.connection_type.name} is not a valid "
                    f"{self.device_type} connection type. Valid connection types: "
                    f"{[x.name for x in VALID_DEVICE_CONNECTION_TYPES[self.device_type]]}"
                )
                raise TypeError(msg)
        except ValueError as error:
            # this is from an invalid enum name
            raise TypeError(*error.args)  # noqa: TRY200,B904

        # Check if a specific device driver is needed to be specified
        if self.connection_type == ConnectionTypes.REST_API and self.device_driver is None:
            msg = (
                f'Device configuration entry "{self}" specified a '
                f"REST_API connection without providing the Python device driver name to use."
            )
            raise ValueError(msg)

        if self.connection_type == ConnectionTypes.SOCKET and self.lan_port is None:
            msg = (
                f'Device configuration entry "{self}" specified a '
                f"socket connection without a port number."
            )
            raise ValueError(msg)

        # validate address
        try:
            if self.connection_type in [
                ConnectionTypes.TCPIP,
                ConnectionTypes.SOCKET,
                ConnectionTypes.REST_API,
            ]:
                validate_address(self.address)
            elif (  # pylint: disable=too-many-boolean-expressions
                (self.connection_type == ConnectionTypes.USB and "-" not in self.address)
                or (self.connection_type == ConnectionTypes.SERIAL and not self.address.isdigit())
                or (
                    self.connection_type == ConnectionTypes.GPIB
                    and not (self.address.isdigit() and 1 <= int(self.address) <= MAX_GPIB_ADDRESS)
                )
            ):
                raise ValueError
        except ValueError as exc:
            msg = f"""Found invalid value of `address={self.address}`
        Must provide an valid address format for the `connection_type={self.connection_type.value}':
            TCPIP: IP address or the hostname.
            SOCKET: IP address or the hostname (must define `lan_port` field).
            REST_API: IP address or the hostname (must define `lan_port` field).
            SERIAL/ASRL: serial COM port number.
            GPIB: address number (1 <= n <= 30).
            USB: use expression format "<model>-<serial_number>" (ex: "MSO24-ABC0123").
                """
            raise ValueError(msg) from exc

        # Validate serial config
        if self.serial_config is not None:
            # make sure serial attributes with integer input are in valid range
            if (
                self.serial_config.baud_rate is not None
                and self.serial_config.baud_rate not in VALID_SERIAL_BAUD
            ):
                msg = (
                    f"Unexpected baud rate of {self.serial_config.baud_rate}, "
                    f"must be in {sorted(VALID_SERIAL_BAUD)}"
                )
                raise ValueError(msg)
            if (
                self.serial_config.data_bits is not None
                and self.serial_config.data_bits not in VALID_SERIAL_DATA_BITS
            ):
                msg = (
                    f"Unexpected data bits of {self.serial_config.data_bits}, "
                    f"must be in {sorted(VALID_SERIAL_DATA_BITS)}"
                )
                raise ValueError(msg)

    def get_address_expression(self) -> str:
        """Return an address expression.

        This is not the resource expression, it is used to help catch duplicate addresses.
        """
        return (
            self.connection_type.value + " " + self.address + ""
            if self.lan_port is None
            else f":{self.lan_port}"
        )

    def get_visa_resource_expression(self) -> str:
        """Construct the VISA resource expression for the device.

        Example visa expressions:

            - TCPIP0::MSO58-3000260::instr0::INSTR
            - USB0::0x0699::0x0522::3000260::INSTR
            - TCPIP0::192.168.0.1::4000::SOCKET
            - ASRL3::INSTR  (serial on COM port 3)

        Returns:
            A string containing the VISA resource expression.

        Raises:
            NotImplementedError: Indicates that the USBTMC lookup dictionary needs to be
                updated to support the current device.
            ValueError: Indicates that the connection type specified in the config is not valid.
        """
        # If USBTMC for the address check if connected
        if self.connection_type == ConnectionTypes.USB:
            full_model, serial = self.address.split("-", 1)
            # local import here to avoid circular imports
            # pylint: disable=import-outside-toplevel,cyclic-import
            from tm_devices.helpers.functions import get_model_series

            model_series = get_model_series(full_model)
            try:
                usbtmc_config = USB_MODEL_ID_LOOKUP[model_series]

                # This commented out block of code can help deal with devices where the specific
                # model number is the USBTMC model_id but the Python driver is shared between
                # multiple model numbers. It is currently not needed, if this is needed in the
                # future this code block can be uncommented.
                # RELIC # if usbtmc_config.model_id == _USBTMC_UNKNOWN_MODEL:
                # RELIC #     usbtmc_config = USBTMCConfiguration(
                # RELIC #         vendor_id=usbtmc_config.vendor_id,
                # RELIC #         model_id="0x" + (full_model.split(" ", maxsplit=1)[-1])[:4],
                # RELIC #     )

                resource_expr = (
                    f"USB0::{usbtmc_config.vendor_id}::{usbtmc_config.model_id}::{serial}::0::INSTR"
                )
            except KeyError as exc:
                msg = (
                    f"USBTMC model ID has not been configured for the "
                    f"{model_series} model series\nconfig = {self}"
                )
                raise NotImplementedError(msg) from exc
        elif self.connection_type == ConnectionTypes.SOCKET:
            resource_expr = f"TCPIP0::{self.address}::{self.lan_port}::SOCKET"
        elif self.connection_type == ConnectionTypes.TCPIP:
            resource_expr = f"TCPIP0::{self.address}::inst0::INSTR"
        elif self.connection_type == ConnectionTypes.GPIB:
            resource_expr = f"GPIB0::{self.address}::INSTR"
        elif self.connection_type == ConnectionTypes.SERIAL:
            resource_expr = f"ASRL{self.address}::INSTR"
        elif self.connection_type == ConnectionTypes.MOCK:  # pragma: no cover
            resource_expr = f"MOCK0::{self.address}::INSTR"
        else:
            msg = (
                f"{self.connection_type.value} is not a valid connection "
                f"type for a VISA resource."
            )
            raise ValueError(msg)
        return resource_expr


@dataclass
class DMConfigOptions(AsDictionaryMixin):
    """Device Management Configuration options."""

    standalone: Optional[bool] = None
    """Optional[bool]: An option indicating to use PyVISA-py's pure Python VISA backend."""
    setup_cleanup: Optional[bool] = None
    """Optional[bool]: An option indicating to run a device's ``cleanup()`` method when opening the connection."""  # noqa: E501
    teardown_cleanup: Optional[bool] = None
    """Optional[bool]: An option indicating to run a device's ``cleanup()`` method when closing the connection."""  # noqa: E501
    verbose_mode: Optional[bool] = None
    """Optional[bool]: A verbosity flag to turn on more printouts to stdout."""
    verbose_visa: Optional[bool] = None
    """Optional[bool]: A verbosity flag to enable extremely verbose VISA logging to stdout."""
    retry_visa_connection: Optional[bool] = None
    """Optional[bool]: A flag to enable retrying the first VISA connection attempt."""

    def __str__(self) -> str:
        """Complete config entry line for an environment variable."""
        return ",".join(
            [
                opt_key.upper()
                for opt_key, opt_val in self.to_dict(ignore_none=True).items()
                if opt_val
            ]
        )


###############################################################################
#                                CONSTANTS
###############################################################################
# don't include this in the __init__
MAX_GPIB_ADDRESS = 30
# don't include this in the __init__
CONFIG_CLASS_STR_PREFIX_MAPPING: Final = MappingProxyType({SerialConfig: "serial_"})
# don't include this in the __init__
VALID_SERIAL_BAUD: Final = frozenset(
    [
        300,
        600,
        1200,
        2400,
        4800,
        9600,
        14400,
        19200,
        28800,
        31250,
        38400,
        57600,
        115200,
    ]
)
# don't include this in the __init__
VALID_SERIAL_DATA_BITS: Final = frozenset([5, 6, 7, 8])

PYVISA_PY_BACKEND: Final = "@py"
"""str: Constant string which indicates to use the pure Python PyVISA-py backend when creating VISA connections."""  # noqa: E501

SYSTEM_DEFAULT_VISA_BACKEND: Final = ""
"""str: Constant string which indicates to use the current system's default VISA backend when creating VISA connections."""  # noqa: E501

PACKAGE_NAME: Final = "tm_devices"
"""str: Constant string with the name of this package."""

VISA_RESOURCE_EXPRESSION_REGEX: Final = re.compile(
    r"^(\w+)(?:::0X\w+)?::([-.\w]+)(?:::(\w+))?(?:::INST0?)?::INSTR?$"
)
"""re.Pattern[str]: A regex pattern used to capture pieces of VISA resource expressions."""

UNIT_TEST_TIMEOUT: Final = 50
"""int: The VISA timeout value to use during unit tests, in milliseconds."""

VALID_DEVICE_CONNECTION_TYPES: Final = MappingProxyType(
    {
        DeviceTypes.AFG: (
            ConnectionTypes.TCPIP,
            ConnectionTypes.USB,
            ConnectionTypes.SOCKET,
            ConnectionTypes.SERIAL,
            ConnectionTypes.GPIB,
        ),
        DeviceTypes.AWG: (
            ConnectionTypes.TCPIP,
            ConnectionTypes.USB,
            ConnectionTypes.SOCKET,
            ConnectionTypes.SERIAL,
            ConnectionTypes.GPIB,
        ),
        DeviceTypes.DAQ: (
            ConnectionTypes.TCPIP,
            ConnectionTypes.USB,
            ConnectionTypes.GPIB,
            ConnectionTypes.SERIAL,
        ),
        DeviceTypes.DMM: (
            ConnectionTypes.TCPIP,
            ConnectionTypes.USB,
            ConnectionTypes.SOCKET,
            ConnectionTypes.SERIAL,
            ConnectionTypes.GPIB,
        ),
        DeviceTypes.MT: (ConnectionTypes.REST_API,),
        DeviceTypes.PSU: (
            ConnectionTypes.TCPIP,
            ConnectionTypes.USB,
            ConnectionTypes.SOCKET,
            ConnectionTypes.SERIAL,
            ConnectionTypes.GPIB,
        ),
        DeviceTypes.SCOPE: (
            ConnectionTypes.TCPIP,
            ConnectionTypes.USB,
            ConnectionTypes.SOCKET,
            ConnectionTypes.SERIAL,
            ConnectionTypes.GPIB,
            ConnectionTypes.MOCK,
        ),
        DeviceTypes.SMU: (
            ConnectionTypes.TCPIP,
            ConnectionTypes.USB,
            ConnectionTypes.SOCKET,
            ConnectionTypes.SERIAL,
            ConnectionTypes.GPIB,
        ),
        DeviceTypes.SS: (
            ConnectionTypes.TCPIP,
            ConnectionTypes.USB,
            ConnectionTypes.SOCKET,
            ConnectionTypes.GPIB,
        ),
    }
)
"""Dict[tm_devices.helpers.DeviceTypes, Tuple[tm_devices.helpers.ConnectionTypes, ...]]: Mapping of each device type to its supported connection types.

Any additions to this class need to be added to the :py:obj:`~tm_devices.helpers.enums.DeviceTypes`
enum as well.
"""  # noqa: E501

# USBTMC configuration defines
_TEKTRONIX_USBTMC_VENDOR_ID: Final = "0x0699"
_KEITHLEY_USBTMC_VENDOR_ID: Final = "0x05E6"
USB_MODEL_ID_LOOKUP: Final[Mapping[str, USBTMCConfiguration]] = MappingProxyType(
    {
        SupportedModels.MDO3K.value: USBTMCConfiguration(
            vendor_id=_TEKTRONIX_USBTMC_VENDOR_ID, model_id="0x0408"
        ),
        SupportedModels.MSO2.value: USBTMCConfiguration(
            vendor_id=_TEKTRONIX_USBTMC_VENDOR_ID, model_id="0x0105"
        ),
        SupportedModels.MSO4.value: USBTMCConfiguration(
            vendor_id=_TEKTRONIX_USBTMC_VENDOR_ID, model_id="0x0527"
        ),
        SupportedModels.MSO5.value: USBTMCConfiguration(
            vendor_id=_TEKTRONIX_USBTMC_VENDOR_ID, model_id="0x0522"
        ),
        SupportedModels.MSO5B.value: USBTMCConfiguration(
            vendor_id=_TEKTRONIX_USBTMC_VENDOR_ID, model_id="0x0522"
        ),
        SupportedModels.MSO5LP.value: USBTMCConfiguration(
            vendor_id=_TEKTRONIX_USBTMC_VENDOR_ID, model_id="0x0529"
        ),
        SupportedModels.MSO6.value: USBTMCConfiguration(
            vendor_id=_TEKTRONIX_USBTMC_VENDOR_ID, model_id="0x0528"
        ),
        SupportedModels.MSO6B.value: USBTMCConfiguration(
            vendor_id=_TEKTRONIX_USBTMC_VENDOR_ID, model_id="0x0530"
        ),
        SupportedModels.LPD6.value: USBTMCConfiguration(
            vendor_id=_TEKTRONIX_USBTMC_VENDOR_ID, model_id="0x052F"
        ),
        SupportedModels.AFG3K.value: USBTMCConfiguration(
            vendor_id=_TEKTRONIX_USBTMC_VENDOR_ID, model_id="0x0345"
        ),
        SupportedModels.SMU2450.value: USBTMCConfiguration(
            vendor_id=_KEITHLEY_USBTMC_VENDOR_ID, model_id="0x2450"
        ),
        SupportedModels.SMU2460.value: USBTMCConfiguration(
            vendor_id=_KEITHLEY_USBTMC_VENDOR_ID, model_id="0x2460"
        ),
        SupportedModels.SMU2461.value: USBTMCConfiguration(
            vendor_id=_KEITHLEY_USBTMC_VENDOR_ID, model_id="0x2461"
        ),
        SupportedModels.SMU2470.value: USBTMCConfiguration(
            vendor_id=_KEITHLEY_USBTMC_VENDOR_ID, model_id="0x2470"
        ),
        SupportedModels.SMU2601A.value: USBTMCConfiguration(
            vendor_id=_KEITHLEY_USBTMC_VENDOR_ID, model_id="0x2601"
        ),
        SupportedModels.SMU2602A.value: USBTMCConfiguration(
            vendor_id=_KEITHLEY_USBTMC_VENDOR_ID, model_id="0x2602"
        ),
        SupportedModels.SMU2604A.value: USBTMCConfiguration(
            vendor_id=_KEITHLEY_USBTMC_VENDOR_ID, model_id="0x2604"
        ),
        SupportedModels.SMU2611A.value: USBTMCConfiguration(
            vendor_id=_KEITHLEY_USBTMC_VENDOR_ID, model_id="0x2611"
        ),
        SupportedModels.SMU2612A.value: USBTMCConfiguration(
            vendor_id=_KEITHLEY_USBTMC_VENDOR_ID, model_id="0x2612"
        ),
        SupportedModels.SMU2614A.value: USBTMCConfiguration(
            vendor_id=_KEITHLEY_USBTMC_VENDOR_ID, model_id="0x2614"
        ),
        SupportedModels.SMU2634A.value: USBTMCConfiguration(
            vendor_id=_KEITHLEY_USBTMC_VENDOR_ID, model_id="0x2634"
        ),
        SupportedModels.SMU2635A.value: USBTMCConfiguration(
            vendor_id=_KEITHLEY_USBTMC_VENDOR_ID, model_id="0x2635"
        ),
        SupportedModels.SMU2636A.value: USBTMCConfiguration(
            vendor_id=_KEITHLEY_USBTMC_VENDOR_ID, model_id="0x2636"
        ),
        SupportedModels.SMU2601B.value: USBTMCConfiguration(
            vendor_id=_KEITHLEY_USBTMC_VENDOR_ID, model_id="0x2601"
        ),
        SupportedModels.SMU2602B.value: USBTMCConfiguration(
            vendor_id=_KEITHLEY_USBTMC_VENDOR_ID, model_id="0x2602"
        ),
        SupportedModels.SMU2604B.value: USBTMCConfiguration(
            vendor_id=_KEITHLEY_USBTMC_VENDOR_ID, model_id="0x2604"
        ),
        SupportedModels.SMU2606B.value: USBTMCConfiguration(
            vendor_id=_KEITHLEY_USBTMC_VENDOR_ID, model_id="0x2606"
        ),
        SupportedModels.SMU2611B.value: USBTMCConfiguration(
            vendor_id=_KEITHLEY_USBTMC_VENDOR_ID, model_id="0x2611"
        ),
        SupportedModels.SMU2612B.value: USBTMCConfiguration(
            vendor_id=_KEITHLEY_USBTMC_VENDOR_ID, model_id="0x2612"
        ),
        SupportedModels.SMU2614B.value: USBTMCConfiguration(
            vendor_id=_KEITHLEY_USBTMC_VENDOR_ID, model_id="0x2614"
        ),
        SupportedModels.SMU2634B.value: USBTMCConfiguration(
            vendor_id=_KEITHLEY_USBTMC_VENDOR_ID, model_id="0x2634"
        ),
        SupportedModels.SMU2635B.value: USBTMCConfiguration(
            vendor_id=_KEITHLEY_USBTMC_VENDOR_ID, model_id="0x2635"
        ),
        SupportedModels.SMU2636B.value: USBTMCConfiguration(
            vendor_id=_KEITHLEY_USBTMC_VENDOR_ID, model_id="0x2636"
        ),
        SupportedModels.PSU2200.value: USBTMCConfiguration(
            vendor_id=_KEITHLEY_USBTMC_VENDOR_ID, model_id="0x2200"
        ),
        SupportedModels.PSU2220.value: USBTMCConfiguration(
            vendor_id=_KEITHLEY_USBTMC_VENDOR_ID, model_id="0x2220"
        ),
        SupportedModels.PSU2230.value: USBTMCConfiguration(
            vendor_id=_KEITHLEY_USBTMC_VENDOR_ID, model_id="0x2230"
        ),
        SupportedModels.PSU2231.value: USBTMCConfiguration(
            vendor_id=_KEITHLEY_USBTMC_VENDOR_ID, model_id="0x2231"
        ),
        SupportedModels.PSU2231A.value: USBTMCConfiguration(
            vendor_id=_KEITHLEY_USBTMC_VENDOR_ID, model_id="0x2231"
        ),
        SupportedModels.PSU2280.value: USBTMCConfiguration(
            vendor_id=_KEITHLEY_USBTMC_VENDOR_ID, model_id="0x2280"
        ),
        SupportedModels.PSU2281.value: USBTMCConfiguration(
            vendor_id=_KEITHLEY_USBTMC_VENDOR_ID, model_id="0x2281"
        ),
        SupportedModels.AWG5200.value: USBTMCConfiguration(
            vendor_id=_TEKTRONIX_USBTMC_VENDOR_ID, model_id="0x0503"
        ),
        SupportedModels.AWG70KA.value: USBTMCConfiguration(
            vendor_id=_TEKTRONIX_USBTMC_VENDOR_ID, model_id="0x0503"
        ),
        SupportedModels.AWG70KB.value: USBTMCConfiguration(
            vendor_id=_TEKTRONIX_USBTMC_VENDOR_ID, model_id="0x0503"
        ),
    }
)
"""Dict[str, tm_devices.helpers.USBTMCConfiguration]: Mapping of model USBTMC info."""



# ruff: noqa: D402,PLR0913
# pyright: reportConstantRedefinition=none
"""The script commands module.

These commands are used in the following models:
SMU2450

THIS FILE IS AUTO-GENERATED, IT SHOULD NOT BE MANUALLY MODIFIED.

Please report an issue if one is found.

Attributes and Functions:

::

    - script.delete()
    - script.load()
"""
from typing import Optional, TYPE_CHECKING

from .._helpers import BaseTSPCmd, NoDeviceProvidedError

if TYPE_CHECKING:
    from tm_devices.drivers.pi.tsp_device import TSPDevice


class Script(BaseTSPCmd):
    """The ``script`` command tree.

    Properties/methods:
        - ``.delete()``: The ``script.delete()`` function.
        - ``.load()``: The ``script.load()`` function.
    """

    def __init__(self, device: Optional["TSPDevice"] = None, cmd_syntax: str = "script") -> None:
        super().__init__(device, cmd_syntax)

    def delete(self, script_name: str) -> None:
        """Run the ``script.delete()`` function.

        **Description:**
            - This function deletes a script from the run-time memory and nonvolatile memory.

        **TSP Syntax:**

        ::

            - script.delete()

        Args:
            script_name: A string that represents the name of the script.

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            self._device.write(  # type: ignore[union-attr]
                f'{self._cmd_syntax}.delete("{script_name}")'
            )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to run the ``{self._cmd_syntax}.delete()`` function."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    def load(self, file: str) -> str:
        """Run the ``script.load()`` function.

        **Description:**
            - This function creates a script from a specified file.

        **TSP Syntax:**

        ::

            - script.load()

        Args:
            file: A string that contains the path and file name of the script file to load; if
                scriptVar is not defined, this name is used as the global variable name for this
                script.

        Returns:
            The result of the function call.

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            return self._device.query(  # type: ignore[union-attr]
                f'print({self._cmd_syntax}.load("{file}"))'
            )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to run the ``{self._cmd_syntax}.load()`` function."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error



"""AFG package init file."""
from tm_devices.drivers.pi.signal_sources.afgs.afg import AFG
from tm_devices.drivers.pi.signal_sources.afgs.afg3k import AFG3K
from tm_devices.drivers.pi.signal_sources.afgs.afg3kb import AFG3KB
from tm_devices.drivers.pi.signal_sources.afgs.afg3kc import AFG3KC
from tm_devices.drivers.pi.signal_sources.afgs.afg31k import AFG31K

__all__ = [
    "AFG",
    "AFG3K",
    "AFG3KB",
    "AFG3KC",
    "AFG31K",
]



"""MSO5KB device driver module."""
from tm_devices.commands import MSO5KBMixin
from tm_devices.drivers.pi.scopes.tekscope_5k_7k_70k.mso5k import MSO5K


class MSO5KB(MSO5KBMixin, MSO5K):
    """MSO5KB device driver."""



"""The calibrate commands module.

These commands are used in the following models:
DPO2K, DPO2KB, MSO2K, MSO2KB

THIS FILE IS AUTO-GENERATED, IT SHOULD NOT BE MANUALLY MODIFIED.

Please report an issue if one is found.

Commands and Queries:

::

    - CALibrate:FACtory {STARt|CONTinue|PREVious|ABOrt|DUmp}
    - CALibrate:FACtory:STATus?
    - CALibrate:INTERNal
    - CALibrate:INTERNal:STARt
    - CALibrate:INTERNal:STATus?
    - CALibrate:POWerupstatus?
    - CALibrate:RESults:FACtory?
    - CALibrate:RESults:SPC?
    - CALibrate:RESults?
    - CALibrate:TEMPerature?
"""
from typing import Optional, TYPE_CHECKING

from .._helpers import SCPICmdRead, SCPICmdWrite, SCPICmdWriteNoArguments

if TYPE_CHECKING:
    from tm_devices.drivers.pi.pi_device import PIDevice


class CalibrateTemperature(SCPICmdRead):
    """The ``CALibrate:TEMPerature`` command.

    **Description:**
        - Returns the oscilloscope temperature at the time the SPC was run.

    **Usage:**
        - Using the ``.query()`` method will send the ``CALibrate:TEMPerature?`` query.
        - Using the ``.verify(value)`` method will send the ``CALibrate:TEMPerature?`` query and
          raise an AssertionError if the returned value does not match ``value``.

    **SCPI Syntax:**

    ::

        - CALibrate:TEMPerature?
    """


class CalibrateResultsSpc(SCPICmdRead):
    """The ``CALibrate:RESults:SPC`` command.

    **Description:**
        - Returns the status of the SPC operation. This query does not initiate a SPC.

    **Usage:**
        - Using the ``.query()`` method will send the ``CALibrate:RESults:SPC?`` query.
        - Using the ``.verify(value)`` method will send the ``CALibrate:RESults:SPC?`` query and
          raise an AssertionError if the returned value does not match ``value``.

    **SCPI Syntax:**

    ::

        - CALibrate:RESults:SPC?
    """


class CalibrateResultsFactory(SCPICmdRead):
    """The ``CALibrate:RESults:FACtory`` command.

    **Description:**
        - Returns the status of internal and factory calibration, without performing any calibration
          operations.

    **Usage:**
        - Using the ``.query()`` method will send the ``CALibrate:RESults:FACtory?`` query.
        - Using the ``.verify(value)`` method will send the ``CALibrate:RESults:FACtory?`` query and
          raise an AssertionError if the returned value does not match ``value``.

    **SCPI Syntax:**

    ::

        - CALibrate:RESults:FACtory?
    """


class CalibrateResults(SCPICmdRead):
    """The ``CALibrate:RESults`` command.

    **Description:**
        - Returns the status of internal and factory calibrations, without performing any
          calibration operations. The results returned do not include the calibration status of
          attached probes. The query is intended to support GO/NoGO testing of the oscilloscope
          calibration readiness: all returned results should indicate PASS status if the
          oscilloscope is 'fit for duty'. It is quite common, however, to use uncalibrated probes
          (particularly when the oscilloscope inputs are connected into a test system with coaxial
          cables).

    **Usage:**
        - Using the ``.query()`` method will send the ``CALibrate:RESults?`` query.
        - Using the ``.verify(value)`` method will send the ``CALibrate:RESults?`` query and raise
          an AssertionError if the returned value does not match ``value``.

    **SCPI Syntax:**

    ::

        - CALibrate:RESults?

    Properties:
        - ``.factory``: The ``CALibrate:RESults:FACtory`` command.
        - ``.spc``: The ``CALibrate:RESults:SPC`` command.
    """

    def __init__(self, device: Optional["PIDevice"], cmd_syntax: str) -> None:
        super().__init__(device, cmd_syntax)
        self._factory = CalibrateResultsFactory(device, f"{self._cmd_syntax}:FACtory")
        self._spc = CalibrateResultsSpc(device, f"{self._cmd_syntax}:SPC")

    @property
    def factory(self) -> CalibrateResultsFactory:
        """Return the ``CALibrate:RESults:FACtory`` command.

        **Description:**
            - Returns the status of internal and factory calibration, without performing any
              calibration operations.

        **Usage:**
            - Using the ``.query()`` method will send the ``CALibrate:RESults:FACtory?`` query.
            - Using the ``.verify(value)`` method will send the ``CALibrate:RESults:FACtory?`` query
              and raise an AssertionError if the returned value does not match ``value``.

        **SCPI Syntax:**

        ::

            - CALibrate:RESults:FACtory?
        """
        return self._factory

    @property
    def spc(self) -> CalibrateResultsSpc:
        """Return the ``CALibrate:RESults:SPC`` command.

        **Description:**
            - Returns the status of the SPC operation. This query does not initiate a SPC.

        **Usage:**
            - Using the ``.query()`` method will send the ``CALibrate:RESults:SPC?`` query.
            - Using the ``.verify(value)`` method will send the ``CALibrate:RESults:SPC?`` query and
              raise an AssertionError if the returned value does not match ``value``.

        **SCPI Syntax:**

        ::

            - CALibrate:RESults:SPC?
        """
        return self._spc


class CalibratePowerupstatus(SCPICmdRead):
    """The ``CALibrate:POWerupstatus`` command.

    **Description:**
        - Returns the status of the oscilloscope following power on.

    **Usage:**
        - Using the ``.query()`` method will send the ``CALibrate:POWerupstatus?`` query.
        - Using the ``.verify(value)`` method will send the ``CALibrate:POWerupstatus?`` query and
          raise an AssertionError if the returned value does not match ``value``.

    **SCPI Syntax:**

    ::

        - CALibrate:POWerupstatus?
    """


class CalibrateInternalStatus(SCPICmdRead):
    """The ``CALibrate:INTERNal:STATus`` command.

    **Description:**
        - This query-only command returns the current status of the signal path calibration.

    **Usage:**
        - Using the ``.query()`` method will send the ``CALibrate:INTERNal:STATus?`` query.
        - Using the ``.verify(value)`` method will send the ``CALibrate:INTERNal:STATus?`` query and
          raise an AssertionError if the returned value does not match ``value``.

    **SCPI Syntax:**

    ::

        - CALibrate:INTERNal:STATus?
    """


class CalibrateInternalStart(SCPICmdWriteNoArguments):
    """The ``CALibrate:INTERNal:STARt`` command.

    **Description:**
        - This command (no query form) starts the signal path calibration (SPC) of the analog
          channels. This command is the same as the ``CALIBRATE:INTERNAL`` command. You can use the
          ``CALIBRATE:INTERNAL:STATUS`` query to return the current status of the signal path
          calibration of the instrument.

    **Usage:**
        - Using the ``.write()`` method will send the ``CALibrate:INTERNal:STARt`` command.

    **SCPI Syntax:**

    ::

        - CALibrate:INTERNal:STARt
    """


class CalibrateInternal(SCPICmdWriteNoArguments, SCPICmdRead):
    """The ``CALibrate:INTERNal`` command.

    **Description:**
        - This command (no query form) starts the signal path calibration (SPC) of the instrument.
          You can use the ``CALIBRATE:INTERNAL:STATUS`` query to return the current status of the
          signal path calibration of the instrument.

    **Usage:**
        - Using the ``.write()`` method will send the ``CALibrate:INTERNal`` command.

    **SCPI Syntax:**

    ::

        - CALibrate:INTERNal

    Properties:
        - ``.start``: The ``CALibrate:INTERNal:STARt`` command.
        - ``.status``: The ``CALibrate:INTERNal:STATus`` command.
    """

    def __init__(self, device: Optional["PIDevice"], cmd_syntax: str) -> None:
        super().__init__(device, cmd_syntax)
        self._start = CalibrateInternalStart(device, f"{self._cmd_syntax}:STARt")
        self._status = CalibrateInternalStatus(device, f"{self._cmd_syntax}:STATus")

    @property
    def start(self) -> CalibrateInternalStart:
        """Return the ``CALibrate:INTERNal:STARt`` command.

        **Description:**
            - This command (no query form) starts the signal path calibration (SPC) of the analog
              channels. This command is the same as the ``CALIBRATE:INTERNAL`` command. You can use
              the ``CALIBRATE:INTERNAL:STATUS`` query to return the current status of the signal
              path calibration of the instrument.

        **Usage:**
            - Using the ``.write()`` method will send the ``CALibrate:INTERNal:STARt`` command.

        **SCPI Syntax:**

        ::

            - CALibrate:INTERNal:STARt
        """
        return self._start

    @property
    def status(self) -> CalibrateInternalStatus:
        """Return the ``CALibrate:INTERNal:STATus`` command.

        **Description:**
            - This query-only command returns the current status of the signal path calibration.

        **Usage:**
            - Using the ``.query()`` method will send the ``CALibrate:INTERNal:STATus?`` query.
            - Using the ``.verify(value)`` method will send the ``CALibrate:INTERNal:STATus?`` query
              and raise an AssertionError if the returned value does not match ``value``.

        **SCPI Syntax:**

        ::

            - CALibrate:INTERNal:STATus?
        """
        return self._status


class CalibrateFactoryStatus(SCPICmdRead):
    """The ``CALibrate:FACtory:STATus`` command.

    **Description:**
        - Returns the factory calibration status value saved in nonvolatile memory.

    **Usage:**
        - Using the ``.query()`` method will send the ``CALibrate:FACtory:STATus?`` query.
        - Using the ``.verify(value)`` method will send the ``CALibrate:FACtory:STATus?`` query and
          raise an AssertionError if the returned value does not match ``value``.

    **SCPI Syntax:**

    ::

        - CALibrate:FACtory:STATus?
    """


class CalibrateFactory(SCPICmdWrite, SCPICmdRead):
    """The ``CALibrate:FACtory`` command.

    **Description:**
        - Provides the controls for starting and stopping the factory calibration process. The
          factory calibration process consists of a series of steps.

    **Usage:**
        - Using the ``.write(value)`` method will send the ``CALibrate:FACtory value`` command.

    **SCPI Syntax:**

    ::

        - CALibrate:FACtory {STARt|CONTinue|PREVious|ABOrt|DUmp}

    **Info:**
        - ``STARt`` initializes the factory calibration sequence and starts the first calibration
          step.
        - ``CONTinue`` begins the next factory calibration step.
        - ``PREVious`` attempts to run the most recent factory calibration step again.
        - ``ABOrt`` stops the calibration process.
        - ``DUmp`` stops the calibration and prints the calibration constants.

    Properties:
        - ``.status``: The ``CALibrate:FACtory:STATus`` command.
    """

    def __init__(self, device: Optional["PIDevice"], cmd_syntax: str) -> None:
        super().__init__(device, cmd_syntax)
        self._status = CalibrateFactoryStatus(device, f"{self._cmd_syntax}:STATus")

    @property
    def status(self) -> CalibrateFactoryStatus:
        """Return the ``CALibrate:FACtory:STATus`` command.

        **Description:**
            - Returns the factory calibration status value saved in nonvolatile memory.

        **Usage:**
            - Using the ``.query()`` method will send the ``CALibrate:FACtory:STATus?`` query.
            - Using the ``.verify(value)`` method will send the ``CALibrate:FACtory:STATus?`` query
              and raise an AssertionError if the returned value does not match ``value``.

        **SCPI Syntax:**

        ::

            - CALibrate:FACtory:STATus?
        """
        return self._status


class Calibrate(SCPICmdRead):
    """The ``CALibrate`` command tree.

    **Usage:**
        - Using the ``.query()`` method will send the ``CALibrate?`` query.
        - Using the ``.verify(value)`` method will send the ``CALibrate?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    Properties:
        - ``.factory``: The ``CALibrate:FACtory`` command.
        - ``.internal``: The ``CALibrate:INTERNal`` command.
        - ``.powerupstatus``: The ``CALibrate:POWerupstatus`` command.
        - ``.results``: The ``CALibrate:RESults`` command.
        - ``.temperature``: The ``CALibrate:TEMPerature`` command.
    """

    def __init__(self, device: Optional["PIDevice"] = None, cmd_syntax: str = "CALibrate") -> None:
        super().__init__(device, cmd_syntax)
        self._factory = CalibrateFactory(device, f"{self._cmd_syntax}:FACtory")
        self._internal = CalibrateInternal(device, f"{self._cmd_syntax}:INTERNal")
        self._powerupstatus = CalibratePowerupstatus(device, f"{self._cmd_syntax}:POWerupstatus")
        self._results = CalibrateResults(device, f"{self._cmd_syntax}:RESults")
        self._temperature = CalibrateTemperature(device, f"{self._cmd_syntax}:TEMPerature")

    @property
    def factory(self) -> CalibrateFactory:
        """Return the ``CALibrate:FACtory`` command.

        **Description:**
            - Provides the controls for starting and stopping the factory calibration process. The
              factory calibration process consists of a series of steps.

        **Usage:**
            - Using the ``.write(value)`` method will send the ``CALibrate:FACtory value`` command.

        **SCPI Syntax:**

        ::

            - CALibrate:FACtory {STARt|CONTinue|PREVious|ABOrt|DUmp}

        **Info:**
            - ``STARt`` initializes the factory calibration sequence and starts the first
              calibration step.
            - ``CONTinue`` begins the next factory calibration step.
            - ``PREVious`` attempts to run the most recent factory calibration step again.
            - ``ABOrt`` stops the calibration process.
            - ``DUmp`` stops the calibration and prints the calibration constants.

        Sub-properties:
            - ``.status``: The ``CALibrate:FACtory:STATus`` command.
        """
        return self._factory

    @property
    def internal(self) -> CalibrateInternal:
        """Return the ``CALibrate:INTERNal`` command.

        **Description:**
            - This command (no query form) starts the signal path calibration (SPC) of the
              instrument. You can use the ``CALIBRATE:INTERNAL:STATUS`` query to return the current
              status of the signal path calibration of the instrument.

        **Usage:**
            - Using the ``.write()`` method will send the ``CALibrate:INTERNal`` command.

        **SCPI Syntax:**

        ::

            - CALibrate:INTERNal

        Sub-properties:
            - ``.start``: The ``CALibrate:INTERNal:STARt`` command.
            - ``.status``: The ``CALibrate:INTERNal:STATus`` command.
        """
        return self._internal

    @property
    def powerupstatus(self) -> CalibratePowerupstatus:
        """Return the ``CALibrate:POWerupstatus`` command.

        **Description:**
            - Returns the status of the oscilloscope following power on.

        **Usage:**
            - Using the ``.query()`` method will send the ``CALibrate:POWerupstatus?`` query.
            - Using the ``.verify(value)`` method will send the ``CALibrate:POWerupstatus?`` query
              and raise an AssertionError if the returned value does not match ``value``.

        **SCPI Syntax:**

        ::

            - CALibrate:POWerupstatus?
        """
        return self._powerupstatus

    @property
    def results(self) -> CalibrateResults:
        """Return the ``CALibrate:RESults`` command.

        **Description:**
            - Returns the status of internal and factory calibrations, without performing any
              calibration operations. The results returned do not include the calibration status of
              attached probes. The query is intended to support GO/NoGO testing of the oscilloscope
              calibration readiness: all returned results should indicate PASS status if the
              oscilloscope is 'fit for duty'. It is quite common, however, to use uncalibrated
              probes (particularly when the oscilloscope inputs are connected into a test system
              with coaxial cables).

        **Usage:**
            - Using the ``.query()`` method will send the ``CALibrate:RESults?`` query.
            - Using the ``.verify(value)`` method will send the ``CALibrate:RESults?`` query and
              raise an AssertionError if the returned value does not match ``value``.

        **SCPI Syntax:**

        ::

            - CALibrate:RESults?

        Sub-properties:
            - ``.factory``: The ``CALibrate:RESults:FACtory`` command.
            - ``.spc``: The ``CALibrate:RESults:SPC`` command.
        """
        return self._results

    @property
    def temperature(self) -> CalibrateTemperature:
        """Return the ``CALibrate:TEMPerature`` command.

        **Description:**
            - Returns the oscilloscope temperature at the time the SPC was run.

        **Usage:**
            - Using the ``.query()`` method will send the ``CALibrate:TEMPerature?`` query.
            - Using the ``.verify(value)`` method will send the ``CALibrate:TEMPerature?`` query and
              raise an AssertionError if the returned value does not match ``value``.

        **SCPI Syntax:**

        ::

            - CALibrate:TEMPerature?
        """
        return self._temperature



# ruff: noqa: D402,PLR0913
# pyright: reportConstantRedefinition=none
"""The digio commands module.

These commands are used in the following models:
DAQ6510, SMU2450, SMU2460, SMU2461, SMU2470

THIS FILE IS AUTO-GENERATED, IT SHOULD NOT BE MANUALLY MODIFIED.

Please report an issue if one is found.

Attributes and Functions:

::

    - digio.line[N].mode
    - digio.line[N].reset()
    - digio.line[N].state
    - digio.readport()
    - digio.writeport()
"""
from typing import Dict, Optional, TYPE_CHECKING, Union

from .._helpers import (
    BaseTSPCmd,
    DefaultDictPassKeyToFactory,
    NoDeviceProvidedError,
    ValidatedDynamicNumberCmd,
)

if TYPE_CHECKING:
    from tm_devices.drivers.pi.tsp_device import TSPDevice


class DigioLineItem(ValidatedDynamicNumberCmd, BaseTSPCmd):
    """The ``digio.line[N]`` command tree.

    **Info:**
        - ``N``, the digital I/O line: 1 to 6.

    Properties/methods:
        - ``.mode``: The ``digio.line[N].mode`` attribute.
        - ``.reset()``: The ``digio.line[N].reset()`` function.
        - ``.state``: The ``digio.line[N].state`` attribute.
    """

    @property
    def mode(self) -> str:
        """Access the ``digio.line[N].mode`` attribute.

        **Description:**
            - This attribute sets the mode of the digital I/O line to be a digital line, trigger
              line, or synchronous line and sets the line to be input, output, or open-drain.

        **Usage:**
            - Accessing this property will send the ``print(digio.line[N].mode)`` query.
            - Setting this property to a value will send the ``digio.line[N].mode = value`` command.

        **TSP Syntax:**

        ::

            - digio.line[N].mode = value
            - print(digio.line[N].mode)

        **Info:**
            - ``N``, the digital I/O line: 1 to 6.

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            if self._device.command_syntax_enabled:  # type: ignore[union-attr]
                return self._cmd_syntax + ".mode"
            return self._device.query(f"print({self._cmd_syntax}.mode)")  # type: ignore[union-attr]
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to access the ``{self._cmd_syntax}.mode`` attribute."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    @mode.setter
    def mode(self, value: Union[str, float]) -> None:
        """Access the ``digio.line[N].mode`` attribute.

        **Description:**
            - This attribute sets the mode of the digital I/O line to be a digital line, trigger
              line, or synchronous line and sets the line to be input, output, or open-drain.

        **Usage:**
            - Accessing this property will send the ``print(digio.line[N].mode)`` query.
            - Setting this property to a value will send the ``digio.line[N].mode = value`` command.

        **TSP Syntax:**

        ::

            - digio.line[N].mode = value
            - print(digio.line[N].mode)

        **Info:**
            - ``N``, the digital I/O line: 1 to 6.

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            if self._device.command_verification_enabled:  # type: ignore[union-attr]
                self._device.set_and_check(  # type: ignore[union-attr]
                    self._cmd_syntax + ".mode", value
                )
            else:
                self._device.write(f"{self._cmd_syntax}.mode = {value}")  # type: ignore[union-attr]
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to access the ``{self._cmd_syntax}.mode`` attribute."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    @property
    def state(self) -> str:
        """Access the ``digio.line[N].state`` attribute.

        **Description:**
            - This function sets a digital I/O line high or low when the line is set for digital
              control and returns the state on the digital I/O lines.

        **Usage:**
            - Accessing this property will send the ``print(digio.line[N].state)`` query.
            - Setting this property to a value will send the ``digio.line[N].state = value``
              command.

        **TSP Syntax:**

        ::

            - digio.line[N].state = value
            - print(digio.line[N].state)

        **Info:**
            - ``N``, the digital I/O line: 1 to 6.

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            if self._device.command_syntax_enabled:  # type: ignore[union-attr]
                return self._cmd_syntax + ".state"
            return self._device.query(  # type: ignore[union-attr]
                f"print({self._cmd_syntax}.state)"
            )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to access the ``{self._cmd_syntax}.state`` attribute."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    @state.setter
    def state(self, value: Union[str, float]) -> None:
        """Access the ``digio.line[N].state`` attribute.

        **Description:**
            - This function sets a digital I/O line high or low when the line is set for digital
              control and returns the state on the digital I/O lines.

        **Usage:**
            - Accessing this property will send the ``print(digio.line[N].state)`` query.
            - Setting this property to a value will send the ``digio.line[N].state = value``
              command.

        **TSP Syntax:**

        ::

            - digio.line[N].state = value
            - print(digio.line[N].state)

        **Info:**
            - ``N``, the digital I/O line: 1 to 6.

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            if self._device.command_verification_enabled:  # type: ignore[union-attr]
                self._device.set_and_check(  # type: ignore[union-attr]
                    self._cmd_syntax + ".state", value
                )
            else:
                self._device.write(  # type: ignore[union-attr]
                    f"{self._cmd_syntax}.state = {value}"
                )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to access the ``{self._cmd_syntax}.state`` attribute."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    def reset(self) -> None:
        """Run the ``digio.line[N].reset()`` function.

        **Description:**
            - This function resets digital I/O line values to their factory defaults.

        **TSP Syntax:**

        ::

            - digio.line[N].reset()

        **Info:**
            - ``N``, the digital I/O line: 1 to 6.

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            self._device.write(f"{self._cmd_syntax}.reset()")  # type: ignore[union-attr]
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to run the ``{self._cmd_syntax}.reset()`` function."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error


class Digio(BaseTSPCmd):
    """The ``digio`` command tree.

    Constants:
        - ``.MODE_DIGITAL_IN``: Set the digital I/O mode to digital control, input.
        - ``.MODE_DIGITAL_OPEN_DRAIN``: Set the digital I/O mode to digital control, open-drain.
        - ``.MODE_DIGITAL_OUT``: Set the digital I/O mode to digital control, output.
        - ``.MODE_SYNCHRONOUS_ACCEPTOR``: Set the digital I/O mode to synchronous acceptor.
        - ``.MODE_SYNCHRONOUS_MASTER``: Set the digital I/O mode to synchronous master.
        - ``.MODE_TRIGGER_IN``: Set the digital I/O mode to trigger control, input.
        - ``.MODE_TRIGGER_OPEN_DRAIN``: Set the digital I/O mode to trigger control, open-drain.
        - ``.MODE_TRIGGER_OUT``: Set the digital I/O mode to trigger control, output.
        - ``.STATE_HIGH``: Set the line high.
        - ``.STATE_LOW``: Set the line low.

    Properties/methods:
        - ``.line``: The ``digio.line[N]`` command tree.
        - ``.readport()``: The ``digio.readport()`` function.
        - ``.writeport()``: The ``digio.writeport()`` function.
    """

    MODE_DIGITAL_IN = "digio.MODE_DIGITAL_IN"
    """str: Set the digital I/O mode to digital control, input."""
    MODE_DIGITAL_OPEN_DRAIN = "digio.MODE_DIGITAL_OPEN_DRAIN"
    """str: Set the digital I/O mode to digital control, open-drain."""
    MODE_DIGITAL_OUT = "digio.MODE_DIGITAL_OUT"
    """str: Set the digital I/O mode to digital control, output."""
    MODE_SYNCHRONOUS_ACCEPTOR = "digio.MODE_SYNCHRONOUS_ACCEPTOR"
    """str: Set the digital I/O mode to synchronous acceptor."""
    MODE_SYNCHRONOUS_MASTER = "digio.MODE_SYNCHRONOUS_MASTER"
    """str: Set the digital I/O mode to synchronous master."""
    MODE_TRIGGER_IN = "digio.MODE_TRIGGER_IN"
    """str: Set the digital I/O mode to trigger control, input."""
    MODE_TRIGGER_OPEN_DRAIN = "digio.MODE_TRIGGER_OPEN_DRAIN"
    """str: Set the digital I/O mode to trigger control, open-drain."""
    MODE_TRIGGER_OUT = "digio.MODE_TRIGGER_OUT"
    """str: Set the digital I/O mode to trigger control, output."""
    STATE_HIGH = "digio.STATE_HIGH"
    """str: Set the line high."""
    STATE_LOW = "digio.STATE_LOW"
    """str: Set the line low."""

    def __init__(self, device: Optional["TSPDevice"] = None, cmd_syntax: str = "digio") -> None:
        super().__init__(device, cmd_syntax)
        self._line: Dict[int, DigioLineItem] = DefaultDictPassKeyToFactory(
            lambda x: DigioLineItem(device, f"{self._cmd_syntax}.line[{x}]")
        )

    @property
    def line(self) -> Dict[int, DigioLineItem]:
        """Return the ``digio.line[N]`` command tree.

        **Info:**
            - ``N``, the digital I/O line: 1 to 6.

        Sub-properties/methods:
            - ``.mode``: The ``digio.line[N].mode`` attribute.
            - ``.reset()``: The ``digio.line[N].reset()`` function.
            - ``.state``: The ``digio.line[N].state`` attribute.
        """
        return self._line

    def readport(self) -> str:
        """Run the ``digio.readport()`` function.

        **Description:**
            - This function reads the digital I/O port.

        **TSP Syntax:**

        ::

            - digio.readport()

        Returns:
            The result of the function call.

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            return self._device.query(  # type: ignore[union-attr]
                f"print({self._cmd_syntax}.readport())"
            )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to run the ``{self._cmd_syntax}.readport()`` function."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    def writeport(self, data: int) -> None:
        """Run the ``digio.writeport()`` function.

        **Description:**
            - This function writes to all digital I/O lines.

        **TSP Syntax:**

        ::

            - digio.writeport()

        Args:
            data: The value to write to the port (0 to 63).

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            self._device.write(f"{self._cmd_syntax}.writeport({data})")  # type: ignore[union-attr]
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to run the ``{self._cmd_syntax}.writeport()`` function."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error



# pylint: disable=line-too-long
# ruff: noqa: D402,PLR0913
# pyright: reportConstantRedefinition=none
"""The tsplink commands module.

These commands are used in the following models:
SMU2601B, SMU2602B, SMU2611B, SMU2612B, SMU2635B, SMU2636B

THIS FILE IS AUTO-GENERATED, IT SHOULD NOT BE MANUALLY MODIFIED.

Please report an issue if one is found.

Attributes and Functions:

::

    - tsplink.group
    - tsplink.master
    - tsplink.node
    - tsplink.readbit()
    - tsplink.readport()
    - tsplink.reset()
    - tsplink.state
    - tsplink.trigger[N].assert()
    - tsplink.trigger[N].clear()
    - tsplink.trigger[N].mode
    - tsplink.trigger[N].overrun
    - tsplink.trigger[N].pulsewidth
    - tsplink.trigger[N].release()
    - tsplink.trigger[N].reset()
    - tsplink.trigger[N].stimulus
    - tsplink.trigger[N].wait()
    - tsplink.writebit()
    - tsplink.writeport()
    - tsplink.writeprotect
"""
from typing import Dict, Optional, TYPE_CHECKING, Union

from .._helpers import (
    BaseTSPCmd,
    DefaultDictPassKeyToFactory,
    NoDeviceProvidedError,
    ValidatedDynamicNumberCmd,
)

if TYPE_CHECKING:
    from tm_devices.drivers.pi.tsp_device import TSPDevice


class TsplinkTriggerItem(ValidatedDynamicNumberCmd, BaseTSPCmd):
    """The ``tsplink.trigger[N]`` command tree.

    **Info:**
        - ``N``, the trigger line (1 to 3).

    Constants:
        - ``.EVENT_ID``: The number that is used for the trigger events.

    Properties/methods:
        - ``.assert()``: The ``tsplink.trigger[N].assert()`` function.
        - ``.clear()``: The ``tsplink.trigger[N].clear()`` function.
        - ``.mode``: The ``tsplink.trigger[N].mode`` attribute.
        - ``.overrun``: The ``tsplink.trigger[N].overrun`` attribute.
        - ``.pulsewidth``: The ``tsplink.trigger[N].pulsewidth`` attribute.
        - ``.release()``: The ``tsplink.trigger[N].release()`` function.
        - ``.reset()``: The ``tsplink.trigger[N].reset()`` function.
        - ``.stimulus``: The ``tsplink.trigger[N].stimulus`` attribute.
        - ``.wait()``: The ``tsplink.trigger[N].wait()`` function.
    """

    EVENT_ID = "tsplink.trigger[N].EVENT_ID"
    """str: The number that is used for the trigger events."""

    def __init__(self, device: Optional["TSPDevice"], cmd_syntax: str) -> None:
        super().__init__(device, cmd_syntax)
        # pylint: disable=invalid-name
        self.EVENT_ID = self.EVENT_ID.replace(
            "[N]", f"[{self._cmd_syntax.rsplit('[', maxsplit=1)[-1].split(']', maxsplit=1)[0]}]"
        )

    @property
    def mode(self) -> str:
        """Access the ``tsplink.trigger[N].mode`` attribute.

        **Description:**
            - This attribute defines the trigger operation and detection mode. This command is not
              available on the 2604B, 2614B, or 2634B.

        **Usage:**
            - Accessing this property will send the ``print(tsplink.trigger[N].mode)`` query.
            - Setting this property to a value will send the ``tsplink.trigger[N].mode = value``
              command.

        **TSP Syntax:**

        ::

            - tsplink.trigger[N].mode = value
            - print(tsplink.trigger[N].mode)

        **Info:**
            - ``N``, the trigger line (1 to 3).

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            if self._device.command_syntax_enabled:  # type: ignore[union-attr]
                return self._cmd_syntax + ".mode"
            return self._device.query(f"print({self._cmd_syntax}.mode)")  # type: ignore[union-attr]
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to access the ``{self._cmd_syntax}.mode`` attribute."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    @mode.setter
    def mode(self, value: Union[str, float]) -> None:
        """Access the ``tsplink.trigger[N].mode`` attribute.

        **Description:**
            - This attribute defines the trigger operation and detection mode. This command is not
              available on the 2604B, 2614B, or 2634B.

        **Usage:**
            - Accessing this property will send the ``print(tsplink.trigger[N].mode)`` query.
            - Setting this property to a value will send the ``tsplink.trigger[N].mode = value``
              command.

        **TSP Syntax:**

        ::

            - tsplink.trigger[N].mode = value
            - print(tsplink.trigger[N].mode)

        **Info:**
            - ``N``, the trigger line (1 to 3).

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            if self._device.command_verification_enabled:  # type: ignore[union-attr]
                self._device.set_and_check(  # type: ignore[union-attr]
                    self._cmd_syntax + ".mode", value
                )
            else:
                self._device.write(f"{self._cmd_syntax}.mode = {value}")  # type: ignore[union-attr]
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to access the ``{self._cmd_syntax}.mode`` attribute."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    @property
    def overrun(self) -> str:
        """Access the ``tsplink.trigger[N].overrun`` attribute.

        **Description:**
            - This attribute indicates if the event detector ignored an event while in the detected
              state. This command is not available on the 2604B, 2614B, or 2634B.

        **Usage:**
            - Accessing this property will send the ``print(tsplink.trigger[N].overrun)`` query.

        **TSP Syntax:**

        ::

            - print(tsplink.trigger[N].overrun)

        **Info:**
            - ``N``, the trigger line (1 to 3).

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            if self._device.command_syntax_enabled:  # type: ignore[union-attr]
                return self._cmd_syntax + ".overrun"
            return self._device.query(  # type: ignore[union-attr]
                f"print({self._cmd_syntax}.overrun)"
            )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to access the ``{self._cmd_syntax}.overrun`` attribute."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    @property
    def pulsewidth(self) -> str:
        """Access the ``tsplink.trigger[N].pulsewidth`` attribute.

        **Description:**
            - This attribute sets the length of time that the trigger line is asserted for output
              triggers. This command is not available on the 2604B, 2614B, or 2634B.

        **Usage:**
            - Accessing this property will send the ``print(tsplink.trigger[N].pulsewidth)`` query.
            - Setting this property to a value will send the
              ``tsplink.trigger[N].pulsewidth = value`` command.

        **TSP Syntax:**

        ::

            - tsplink.trigger[N].pulsewidth = value
            - print(tsplink.trigger[N].pulsewidth)

        **Info:**
            - ``width``, the pulse width (in seconds).
            - ``N``, the trigger line (1 to 3).

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            if self._device.command_syntax_enabled:  # type: ignore[union-attr]
                return self._cmd_syntax + ".pulsewidth"
            return self._device.query(  # type: ignore[union-attr]
                f"print({self._cmd_syntax}.pulsewidth)"
            )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to access the ``{self._cmd_syntax}.pulsewidth`` attribute."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    @pulsewidth.setter
    def pulsewidth(self, value: Union[str, float]) -> None:
        """Access the ``tsplink.trigger[N].pulsewidth`` attribute.

        **Description:**
            - This attribute sets the length of time that the trigger line is asserted for output
              triggers. This command is not available on the 2604B, 2614B, or 2634B.

        **Usage:**
            - Accessing this property will send the ``print(tsplink.trigger[N].pulsewidth)`` query.
            - Setting this property to a value will send the
              ``tsplink.trigger[N].pulsewidth = value`` command.

        **TSP Syntax:**

        ::

            - tsplink.trigger[N].pulsewidth = value
            - print(tsplink.trigger[N].pulsewidth)

        **Info:**
            - ``width``, the pulse width (in seconds).
            - ``N``, the trigger line (1 to 3).

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            if self._device.command_verification_enabled:  # type: ignore[union-attr]
                self._device.set_and_check(  # type: ignore[union-attr]
                    self._cmd_syntax + ".pulsewidth", value
                )
            else:
                self._device.write(  # type: ignore[union-attr]
                    f"{self._cmd_syntax}.pulsewidth = {value}"
                )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to access the ``{self._cmd_syntax}.pulsewidth`` attribute."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    @property
    def stimulus(self) -> str:
        """Access the ``tsplink.trigger[N].stimulus`` attribute.

        **Description:**
            - This attribute specifies the event that causes the synchronization line to assert a
              trigger. This command is not available on the 2604B, 2614B, or 2634B.

        **Usage:**
            - Accessing this property will send the ``print(tsplink.trigger[N].stimulus)`` query.
            - Setting this property to a value will send the ``tsplink.trigger[N].stimulus = value``
              command.

        **TSP Syntax:**

        ::

            - tsplink.trigger[N].stimulus = value
            - print(tsplink.trigger[N].stimulus)

        **Info:**
            - ``N``, the trigger line (1 to 3).

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            if self._device.command_syntax_enabled:  # type: ignore[union-attr]
                return self._cmd_syntax + ".stimulus"
            return self._device.query(  # type: ignore[union-attr]
                f"print({self._cmd_syntax}.stimulus)"
            )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to access the ``{self._cmd_syntax}.stimulus`` attribute."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    @stimulus.setter
    def stimulus(self, value: Union[str, float]) -> None:
        """Access the ``tsplink.trigger[N].stimulus`` attribute.

        **Description:**
            - This attribute specifies the event that causes the synchronization line to assert a
              trigger. This command is not available on the 2604B, 2614B, or 2634B.

        **Usage:**
            - Accessing this property will send the ``print(tsplink.trigger[N].stimulus)`` query.
            - Setting this property to a value will send the ``tsplink.trigger[N].stimulus = value``
              command.

        **TSP Syntax:**

        ::

            - tsplink.trigger[N].stimulus = value
            - print(tsplink.trigger[N].stimulus)

        **Info:**
            - ``N``, the trigger line (1 to 3).

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            if self._device.command_verification_enabled:  # type: ignore[union-attr]
                self._device.set_and_check(  # type: ignore[union-attr]
                    self._cmd_syntax + ".stimulus", value
                )
            else:
                self._device.write(  # type: ignore[union-attr]
                    f"{self._cmd_syntax}.stimulus = {value}"
                )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to access the ``{self._cmd_syntax}.stimulus`` attribute."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    def assert_(self) -> None:
        """Run the ``tsplink.trigger[N].assert()`` function.

        **Description:**
            - This function simulates the occurrence of the trigger and generates the corresponding
              event ID. This command is not available on the 2604B, 2614B, or 2634B.

        **TSP Syntax:**

        ::

            - tsplink.trigger[N].assert()

        **Info:**
            - ``N``, the trigger line (1 to 3).

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            self._device.write(f"{self._cmd_syntax}.assert()")  # type: ignore[union-attr]
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to run the ``{self._cmd_syntax}.assert()`` function."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    def clear(self) -> None:
        """Run the ``tsplink.trigger[N].clear()`` function.

        **Description:**
            - This function clears the event detector for a LAN trigger. This command is not
              available on the 2604B, 2614B, or 2634B.

        **TSP Syntax:**

        ::

            - tsplink.trigger[N].clear()

        **Info:**
            - ``N``, the trigger line (1 to 3) to clear.

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            self._device.write(f"{self._cmd_syntax}.clear()")  # type: ignore[union-attr]
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to run the ``{self._cmd_syntax}.clear()`` function."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    def release(self) -> None:
        """Run the ``tsplink.trigger[N].release()`` function.

        **Description:**
            - This function releases a latched trigger on the given TSP-Link trigger line. This
              command is not available on the 2604B, 2614B, or 2634B.

        **TSP Syntax:**

        ::

            - tsplink.trigger[N].release()

        **Info:**
            - ``N``, the trigger line (1 to 3).

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            self._device.write(f"{self._cmd_syntax}.release()")  # type: ignore[union-attr]
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to run the ``{self._cmd_syntax}.release()`` function."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    def reset(self) -> None:
        """Run the ``tsplink.trigger[N].reset()`` function.

        **Description:**
            - This function resets some of the TSP-Link trigger attributes to their factory
              defaults. This command is not available on the 2604B, 2614B, or 2634B.

        **TSP Syntax:**

        ::

            - tsplink.trigger[N].reset()

        **Info:**
            - ``N``, the trigger line (1 to 3).

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            self._device.write(f"{self._cmd_syntax}.reset()")  # type: ignore[union-attr]
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to run the ``{self._cmd_syntax}.reset()`` function."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    def wait(self, timeout: float) -> str:
        """Run the ``tsplink.trigger[N].wait()`` function.

        **Description:**
            - This function waits for a trigger. This command is not available on the 2604B, 2614B,
              or 2634B.

        **TSP Syntax:**

        ::

            - tsplink.trigger[N].wait()

        Args:
            timeout: The timeout value in seconds.

        Returns:
            The result of the function call.

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            return self._device.query(  # type: ignore[union-attr]
                f"print({self._cmd_syntax}.wait({timeout}))"
            )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to run the ``{self._cmd_syntax}.wait()`` function."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error


class Tsplink(BaseTSPCmd):
    """The ``tsplink`` command tree.

    Constants:
        - ``.TRIG_BYPASS``: Allows direct control of the line as a digital I/O line.
        - ``.TRIG_EITHER``: Detects rising-edge or falling-edge triggers as input. Asserts a TTL-low
          pulse for output.
        - ``.TRIG_FALLING``: Detects falling-edge triggers as input. Asserts a TTL-low pulse for
          output.
        - ``.TRIG_RISING``: If the programmed state of the line is high, the tsplink.TRIG_RISING
          mode behaves similarly to tsplink.TRIG_RISINGA.
          If the programmed state of the line is low, the tsplink.TRIG_RISING mode behaves similarly to
          tsplink.TRIG_RISINGM.
          Use tsplink.TRIG_RISINGA if the line is in the high output state.
          Use tsplink.TRIG_RISINGM if the line is in the low output state.
        - ``.TRIG_RISINGA``: Detects rising-edge triggers as input. Asserts a TTL-low pulse for
          output.
        - ``.TRIG_RISINGM``: Edge detection as an input is not available. Generates a TTL-high pulse
          as an output trigger.
        - ``.TRIG_SYNCHRONOUS``: Detects the falling-edge input triggers and automatically latches
          and drives the trigger line low. Asserts a TTL-low pulse as an output trigger.
        - ``.TRIG_SYNCHRONOUSA``: Detects the falling-edge input triggers and automatically latches
          and drives the trigger line low.
        - ``.TRIG_SYNCHRONOUSM``: Detects rising-edge triggers as an input. Asserts a TTL-low pulse
          for output.

    Properties/methods:
        - ``.group``: The ``tsplink.group`` attribute.
        - ``.master``: The ``tsplink.master`` attribute.
        - ``.node``: The ``tsplink.node`` attribute.
        - ``.readbit()``: The ``tsplink.readbit()`` function.
        - ``.readport()``: The ``tsplink.readport()`` function.
        - ``.reset()``: The ``tsplink.reset()`` function.
        - ``.state``: The ``tsplink.state`` attribute.
        - ``.trigger``: The ``tsplink.trigger[N]`` command tree.
        - ``.writebit()``: The ``tsplink.writebit()`` function.
        - ``.writeport()``: The ``tsplink.writeport()`` function.
        - ``.writeprotect``: The ``tsplink.writeprotect`` attribute.
    """  # noqa: E501

    TRIG_BYPASS = "tsplink.TRIG_BYPASS"
    """str: Allows direct control of the line as a digital I/O line."""
    TRIG_EITHER = "tsplink.TRIG_EITHER"
    """str: Detects rising-edge or falling-edge triggers as input. Asserts a TTL-low pulse for output."""  # noqa: E501
    TRIG_FALLING = "tsplink.TRIG_FALLING"
    """str: Detects falling-edge triggers as input. Asserts a TTL-low pulse for output."""
    TRIG_RISING = "tsplink.TRIG_RISING"
    """str: If the programmed state of the line is high, the tsplink.TRIG_RISING mode behaves similarly to tsplink.TRIG_RISINGA.
If the programmed state of the line is low, the tsplink.TRIG_RISING mode behaves similarly to tsplink.TRIG_RISINGM.
Use tsplink.TRIG_RISINGA if the line is in the high output state.
Use tsplink.TRIG_RISINGM if the line is in the low output state."""  # noqa: E501
    TRIG_RISINGA = "tsplink.TRIG_RISINGA"
    """str: Detects rising-edge triggers as input. Asserts a TTL-low pulse for output."""
    TRIG_RISINGM = "tsplink.TRIG_RISINGM"
    """str: Edge detection as an input is not available. Generates a TTL-high pulse as an output trigger."""  # noqa: E501
    TRIG_SYNCHRONOUS = "tsplink.TRIG_SYNCHRONOUS"
    """str: Detects the falling-edge input triggers and automatically latches and drives the trigger line low. Asserts a TTL-low pulse as an output trigger."""  # noqa: E501
    TRIG_SYNCHRONOUSA = "tsplink.TRIG_SYNCHRONOUSA"
    """str: Detects the falling-edge input triggers and automatically latches and drives the trigger line low."""  # noqa: E501
    TRIG_SYNCHRONOUSM = "tsplink.TRIG_SYNCHRONOUSM"
    """str: Detects rising-edge triggers as an input. Asserts a TTL-low pulse for output."""

    def __init__(self, device: Optional["TSPDevice"] = None, cmd_syntax: str = "tsplink") -> None:
        super().__init__(device, cmd_syntax)
        self._trigger: Dict[int, TsplinkTriggerItem] = DefaultDictPassKeyToFactory(
            lambda x: TsplinkTriggerItem(device, f"{self._cmd_syntax}.trigger[{x}]")
        )

    @property
    def group(self) -> str:
        """Access the ``tsplink.group`` attribute.

        **Description:**
            - This attribute contains the group number of a TSP-Link node. This command is not
              available on the 2604B, 2614B, or 2634B.

        **Usage:**
            - Accessing this property will send the ``print(tsplink.group)`` query.
            - Setting this property to a value will send the ``tsplink.group = value`` command.

        **TSP Syntax:**

        ::

            - tsplink.group = value
            - print(tsplink.group)

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            if self._device.command_syntax_enabled:  # type: ignore[union-attr]
                return self._cmd_syntax + ".group"
            return self._device.query(  # type: ignore[union-attr]
                f"print({self._cmd_syntax}.group)"
            )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to access the ``{self._cmd_syntax}.group`` attribute."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    @group.setter
    def group(self, value: Union[str, float]) -> None:
        """Access the ``tsplink.group`` attribute.

        **Description:**
            - This attribute contains the group number of a TSP-Link node. This command is not
              available on the 2604B, 2614B, or 2634B.

        **Usage:**
            - Accessing this property will send the ``print(tsplink.group)`` query.
            - Setting this property to a value will send the ``tsplink.group = value`` command.

        **TSP Syntax:**

        ::

            - tsplink.group = value
            - print(tsplink.group)

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            if self._device.command_verification_enabled:  # type: ignore[union-attr]
                self._device.set_and_check(  # type: ignore[union-attr]
                    self._cmd_syntax + ".group", value
                )
            else:
                self._device.write(  # type: ignore[union-attr]
                    f"{self._cmd_syntax}.group = {value}"
                )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to access the ``{self._cmd_syntax}.group`` attribute."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    @property
    def master(self) -> str:
        """Access the ``tsplink.master`` attribute.

        **Description:**
            - This attribute reads the node number assigned to the master node. This command is not
              available on the 2604B, 2614B, or 2634B.

        **Usage:**
            - Accessing this property will send the ``print(tsplink.master)`` query.

        **TSP Syntax:**

        ::

            - print(tsplink.master)

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            if self._device.command_syntax_enabled:  # type: ignore[union-attr]
                return self._cmd_syntax + ".master"
            return self._device.query(  # type: ignore[union-attr]
                f"print({self._cmd_syntax}.master)"
            )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to access the ``{self._cmd_syntax}.master`` attribute."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    @property
    def node(self) -> str:
        """Access the ``tsplink.node`` attribute.

        **Description:**
            - This attribute defines the node number. This command is not available on the 2604B,
              2614B, or 2634B.

        **Usage:**
            - Accessing this property will send the ``print(tsplink.node)`` query.
            - Setting this property to a value will send the ``tsplink.node = value`` command.

        **TSP Syntax:**

        ::

            - tsplink.node = value
            - print(tsplink.node)

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            if self._device.command_syntax_enabled:  # type: ignore[union-attr]
                return self._cmd_syntax + ".node"
            return self._device.query(f"print({self._cmd_syntax}.node)")  # type: ignore[union-attr]
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to access the ``{self._cmd_syntax}.node`` attribute."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    @node.setter
    def node(self, value: Union[str, float]) -> None:
        """Access the ``tsplink.node`` attribute.

        **Description:**
            - This attribute defines the node number. This command is not available on the 2604B,
              2614B, or 2634B.

        **Usage:**
            - Accessing this property will send the ``print(tsplink.node)`` query.
            - Setting this property to a value will send the ``tsplink.node = value`` command.

        **TSP Syntax:**

        ::

            - tsplink.node = value
            - print(tsplink.node)

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            if self._device.command_verification_enabled:  # type: ignore[union-attr]
                self._device.set_and_check(  # type: ignore[union-attr]
                    self._cmd_syntax + ".node", value
                )
            else:
                self._device.write(f"{self._cmd_syntax}.node = {value}")  # type: ignore[union-attr]
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to access the ``{self._cmd_syntax}.node`` attribute."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    @property
    def state(self) -> str:
        """Access the ``tsplink.state`` attribute.

        **Description:**
            - This attribute describes the TSP-Link online state. This command is not available on
              the 2604B, 2614B, or 2634B.

        **Usage:**
            - Accessing this property will send the ``print(tsplink.state)`` query.

        **TSP Syntax:**

        ::

            - print(tsplink.state)

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            if self._device.command_syntax_enabled:  # type: ignore[union-attr]
                return self._cmd_syntax + ".state"
            return self._device.query(  # type: ignore[union-attr]
                f"print({self._cmd_syntax}.state)"
            )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to access the ``{self._cmd_syntax}.state`` attribute."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    @property
    def trigger(self) -> Dict[int, TsplinkTriggerItem]:
        """Return the ``tsplink.trigger[N]`` command tree.

        **Info:**
            - ``N``, the trigger line (1 to 3).

        Constants:
            - ``.EVENT_ID``: The number that is used for the trigger events.

        Sub-properties/methods:
            - ``.assert()``: The ``tsplink.trigger[N].assert()`` function.
            - ``.clear()``: The ``tsplink.trigger[N].clear()`` function.
            - ``.mode``: The ``tsplink.trigger[N].mode`` attribute.
            - ``.overrun``: The ``tsplink.trigger[N].overrun`` attribute.
            - ``.pulsewidth``: The ``tsplink.trigger[N].pulsewidth`` attribute.
            - ``.release()``: The ``tsplink.trigger[N].release()`` function.
            - ``.reset()``: The ``tsplink.trigger[N].reset()`` function.
            - ``.stimulus``: The ``tsplink.trigger[N].stimulus`` attribute.
            - ``.wait()``: The ``tsplink.trigger[N].wait()`` function.
        """
        return self._trigger

    @property
    def writeprotect(self) -> str:
        """Access the ``tsplink.writeprotect`` attribute.

        **Description:**
            - This attribute contains the write-protect mask that protects bits from changes by the
              tsplink.writebit() and tsplink.writeport() functions. This command is not available on
              the 2604B, 2614B, or 2634B.

        **Usage:**
            - Accessing this property will send the ``print(tsplink.writeprotect)`` query.
            - Setting this property to a value will send the ``tsplink.writeprotect = value``
              command.

        **TSP Syntax:**

        ::

            - tsplink.writeprotect = value
            - print(tsplink.writeprotect)

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            if self._device.command_syntax_enabled:  # type: ignore[union-attr]
                return self._cmd_syntax + ".writeprotect"
            return self._device.query(  # type: ignore[union-attr]
                f"print({self._cmd_syntax}.writeprotect)"
            )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to access the ``{self._cmd_syntax}.writeprotect`` attribute."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    @writeprotect.setter
    def writeprotect(self, value: Union[str, float]) -> None:
        """Access the ``tsplink.writeprotect`` attribute.

        **Description:**
            - This attribute contains the write-protect mask that protects bits from changes by the
              tsplink.writebit() and tsplink.writeport() functions. This command is not available on
              the 2604B, 2614B, or 2634B.

        **Usage:**
            - Accessing this property will send the ``print(tsplink.writeprotect)`` query.
            - Setting this property to a value will send the ``tsplink.writeprotect = value``
              command.

        **TSP Syntax:**

        ::

            - tsplink.writeprotect = value
            - print(tsplink.writeprotect)

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            if self._device.command_verification_enabled:  # type: ignore[union-attr]
                self._device.set_and_check(  # type: ignore[union-attr]
                    self._cmd_syntax + ".writeprotect", value
                )
            else:
                self._device.write(  # type: ignore[union-attr]
                    f"{self._cmd_syntax}.writeprotect = {value}"
                )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to access the ``{self._cmd_syntax}.writeprotect`` attribute."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    def readbit(self, n: int) -> str:
        """Run the ``tsplink.readbit()`` function.

        **Description:**
            - This function reads the state of a TSP-Link synchronization line. This command is not
              available on the 2604B, 2614B, or 2634B.

        **TSP Syntax:**

        ::

            - tsplink.readbit()

        Args:
            n: The trigger line (1 to 3).

        Returns:
            The result of the function call.

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            return self._device.query(  # type: ignore[union-attr]
                f"print({self._cmd_syntax}.readbit({n}))"
            )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to run the ``{self._cmd_syntax}.readbit()`` function."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    def readport(self) -> str:
        """Run the ``tsplink.readport()`` function.

        **Description:**
            - This function reads the TSP-Link trigger lines as a digital I/O port. This command is
              not available on the 2604B, 2614B, or 2634B.

        **TSP Syntax:**

        ::

            - tsplink.readport()

        Returns:
            The result of the function call.

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            return self._device.query(  # type: ignore[union-attr]
                f"print({self._cmd_syntax}.readport())"
            )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to run the ``{self._cmd_syntax}.readport()`` function."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    def reset(self, expected_nodes: Optional[int] = None) -> str:
        """Run the ``tsplink.reset()`` function.

        **Description:**
            - This function initializes (resets) all nodes (instruments) in the TSP-Link system.
              This command is not available on the 2604B, 2614B, or 2634B.

        **TSP Syntax:**

        ::

            - tsplink.reset()

        Args:
            expected_nodes (optional): The number of nodes expected on the system (1 to 64).

        Returns:
            The result of the function call.

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            function_args = ", ".join(str(x) for x in (expected_nodes,) if x is not None)
            return self._device.query(  # type: ignore[union-attr]
                f"print({self._cmd_syntax}.reset({function_args}))"
            )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to run the ``{self._cmd_syntax}.reset()`` function."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    def writebit(self, n: int, data: int) -> None:
        """Run the ``tsplink.writebit()`` function.

        **Description:**
            - This function sets a TSP-Link trigger line high or low. This command is not available
              on the 2604B, 2614B, or 2634B.

        **TSP Syntax:**

        ::

            - tsplink.writebit()

        Args:
            n: The trigger line (1 to 3).
            data: The value to write to the bit.

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            self._device.write(  # type: ignore[union-attr]
                f"{self._cmd_syntax}.writebit({n}, {data})"
            )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to run the ``{self._cmd_syntax}.writebit()`` function."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    def writeport(self, data: int) -> None:
        """Run the ``tsplink.writeport()`` function.

        **Description:**
            - This function writes to all TSP-Link synchronization lines. This command is not
              available on the 2604B, 2614B, or 2634B.

        **TSP Syntax:**

        ::

            - tsplink.writeport()

        Args:
            data: Value to write to the port (0 to 7).

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            self._device.write(f"{self._cmd_syntax}.writeport({data})")  # type: ignore[union-attr]
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to run the ``{self._cmd_syntax}.writeport()`` function."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error
